import os
from langchain_openai import ChatOpenAI
from langchain_mistralai import ChatMistralAI
from langchain_core.messages import HumanMessage, AIMessage, SystemMessage
from .models import Configuration

def get_model_to_invoke(config: dict):
    model, api_key, provider = config["model"], config["api_key"], config["provider"]
    model_to_invoke = None

    if provider == 'openai':
        os.environ["OPENAI_API_KEY"] = api_key
        model_to_invoke = ChatOpenAI(model=model)
    elif provider == 'mistralai':
        os.environ["MISTRAL_API_KEY"] = api_key
        model_to_invoke = ChatMistralAI(model=model)

    return model_to_invoke


def call(prompt: str, config: dict):
    model_to_invoke = get_model_to_invoke(config)

    # Messages
    messages_langchain = []
    messages_langchain.append(SystemMessage(content=config["system"]))
    messages_langchain.append(HumanMessage(content=prompt))

    response = model_to_invoke.invoke(messages_langchain)

    return response



import json
from pathlib import Path

config_dir = Path.home() / ".prompt_llm"
config_file = config_dir / "config.json"

def add_to_config(key:str, value: str):
    """Set config such as api_key"""
    # Ensure the directory exists
    config_dir.mkdir(parents=True, exist_ok=True)

    # Load existing config if it exists
    if config_file.exists():
        with open(config_file, "r") as file:
            config_data = json.load(file)
    else:
        config_data = {}

    # Add or update the API key
    config_data[key] = value

    # Save the updated config
    with open(config_file, "w") as file:
        json.dump(config_data, file, indent=4)

def remove_from_config(key:str):
    """Remove config such as api_key"""
    # Ensure the directory exists
    config_dir.mkdir(parents=True, exist_ok=True)

    # Load existing config if it exists
    if config_file.exists():
        with open(config_file, "r") as file:
            config_data = json.load(file)
            if key in config_data:
                del config_data[key]

    # Save the updated config
    with open(config_file, "w") as file:
        json.dump(config_data, file, indent=4)

def load_config():
    # Load existing config if it exists
    if config_file.exists():
        with open(config_file, "r") as file:
            app_config = json.load(file)

    return app_config



#!/usr/bin/env python

import typer
import json
import os

from .utils import add_to_config, remove_from_config, load_config
from .langchain_util import call

app = typer.Typer()

@app.command()
def add_config(api_key: str='', system: str = '', temperature:float = None):
    if api_key:
        add_to_config('api_key', api_key)

    if system:
        add_to_config('system', system)
    
    if temperature is not None:
        add_to_config('temperature', temperature)

@app.command()
def rm_config(key: str):    
    remove_from_config(key)


@app.command()
async def mistralai(prompt: str):
    app_config = load_config()

    if prompt:
        app_config["provider"] = "mistralai"
        app_config["model"] = "mistral-large-latest"

        response = await call(prompt, app_config)

        print(response)

@app.command()
def openai(prompt: str):
    """Tool to prompt openai"""
    app_config = load_config()

    if prompt:
        app_config["provider"] = "openai"
        app_config["model"] = "gpt-4o"

        response = call(prompt, app_config)

        print(response.content)


@app.command()
def version():
    """Retrieve the version from the __init__.py file."""
    package_dir = os.path.dirname(os.path.abspath(__file__))
    init_file = os.path.join(package_dir, '__init__.py')

    with open(init_file, 'r') as file:
        for line in file:
            if line.startswith('__version__'):
                # Extract version from the line
                version = line.split('=')[-1].strip().strip('"').strip("'")
                print(version)

if __name__ == "__main__":
    app()
